<?php

/**
 * @encoding     UTF-8
 * @package      WordPress Demo Content Manager
 * @link         https://codecanyon.net/item/wordpress-demo-content-manager/20669604
 * @copyright    Copyright (C) 2018 Torbara (http://torbara.com). All rights reserved.
 * @license      Envato Standard License http://themeforest.net/licenses/standard?ref=torbara
 * @author       Vladislav Dolgolenko <vladislavdolgolenko.com>
 * @support      <help@vladislavdolgolenko.com>
 */

namespace DM;

use DM\WPObjects\EventManager\ListenerInterface;
use DM\Data;
use DM\Manager;
use DM\Processing\Activator;
use DM\Processing\Installer;
use DM\Log;
use DM\Processing\Statuses;

class AutoReinstallation implements ListenerInterface
{
    public function attach()
    {
        \add_action('wp_loaded', array($this, 'process'));
    }
    
    public function detach()
    {
        \remoute_action('wp_loaded', array($this, 'process'));
    }
    
    public function process()
    {
        global $table_prefix;
        global $wpdb;
        
        $demo_id = Data::getData('origin_demo_id');
        if (!$demo_id) {
            return;
        }
        
        $temporary_demo_id = 'wp_';
        $Manager = Manager::getInstance();
        $OriginDemo = $Manager->getById($demo_id, Manager::REMOUTE);
        $LocalDemo = $Manager->getById($demo_id, Manager::LOCAL);
        $TemporaryDemo = $Manager->getById($temporary_demo_id, Manager::INSTALLED);
        
        // Configuration to processing
        if (!$OriginDemo || !$LocalDemo || !$TemporaryDemo || !$OriginDemo->getDateCreation()) {
            return false;
        }
        
        // Reasone to update
        if ($OriginDemo->isUploaded() && $LocalDemo->getDateCreation() >= $OriginDemo->getDateCreation()) {
            return;
        }
        
        $Log = new Log();
        $Statuses = Statuses::getInstance();
        
        try {
            
            // Do not start when now processing or has not anded another process
            if ($Statuses->isProcessing() === true || $Statuses->getProcessId()) {
                return;
            }
            $Statuses->createPID();
        
            // When demo active, activate temporary demo
            if ($TemporaryDemo->isActivated() === false) {
                $Activator = new Activator($Log);
                $Activator->setDemo($TemporaryDemo);
                $Activator->processing();
                $table_prefix = $TemporaryDemo->getTablePrefix();
                $wpdb->set_prefix($table_prefix);
            }

            // When demo installed -> deinstalling
            if ($LocalDemo->isInstalled()) {
                $Installed = new Installer($Log);
                $Installed->setDemo($LocalDemo);
                $Installed->aborting();
            }

            // when demo not installed -> installing from repository version
            if ($OriginDemo->isInstalled() === false) {
                $Installed = new Installer($Log);
                $Installed->setDemo($OriginDemo);
                $Installed->processing();
            }
            
            if ($OriginDemo->isInstalled()) {
                $Activator = new Activator($Log);
                $Activator->setDemo($OriginDemo);
                $Activator->processing();
                $table_prefix = $OriginDemo->getTablePrefix();
                $wpdb->set_prefix($table_prefix);
            }
            
            /*
            echo 'Demo update!';
            echo '<meta http-equiv="refresh" content="0; url='. get_site_url()  .'">';
            die('');*/
            
        } catch (\Exception $e) {
            $Log->addLog($e->getMessage(), true);
        }
        
    }
}