<?php

/**
 * @encoding     UTF-8
 * @package      WordPress Demo Content Manager
 * @link         https://codecanyon.net/item/wordpress-demo-content-manager/20669604
 * @copyright    Copyright (C) 2018 Torbara (http://torbara.com). All rights reserved.
 * @license      Envato Standard License http://themeforest.net/licenses/standard?ref=torbara
 * @author       Vladislav Dolgolenko <vladislavdolgolenko.com>
 * @support      <help@vladislavdolgolenko.com>
 */

namespace DM\Processing;

use DM\Demo;

abstract class AbstractProcess
{
    /**
     * @var dm_Demo
     */
    protected $Demo = null;
    
    /**
     * @var dm_Log
     */
    protected $Log = null;
    
    protected $steps = array(
    );
    
    public function __construct($Log = null)
    {
        if (!$Log) {
            $this->Log = new dm_Log();
        } else {
            $this->Log = $Log;
        }
    }
    
    abstract public function getProcessName();
    
    abstract public function getProcessId();
    
    /**
     * Starting or continuing processing of demo content installation
     * 
     * @throws Exception
     */
    abstract public function processing();
    
    /**
     * Steps processing
     */
    protected function doSteps()
    {
        $Log = $this->getLog();
        $Demo = $this->getDemo();
        foreach ($this->steps as $key => $step_class_name) {
            $step_status = $key + 1;
            $Step = new $step_class_name($Log, $Demo);
            $Step->setStepStatus($step_status);
            $Step->setProcess($this);
            $Step->doStep();
        }
    }
    
    /**
     * Aborting processing of demo content installation 
     * 
     * @return boolean
     */
    abstract public function aborting();
    
    protected function doAborts()
    {
        $Log = $this->getLog();
        $Demo = $this->getDemo();
        $reverse_steps = array_reverse($this->steps);
        foreach ($reverse_steps as $key => $step_class_name) {
            $step_status = count($reverse_steps) - $key;
            $Step = new $step_class_name($Log, $Demo);
            $Step->setStepStatus($step_status);
            $Step->setProcess($this);
            $Step->doAbort();
        }
    }
    
    protected function start()
    {
        $Statuses = Statuses::getInstance();
        $Statuses->setProcessingStepsCount(count($this->steps));
        $Statuses->setProcessingDemoId($this->getDemo()->getId());
        $Statuses->setProcessName($this->getProcessName());
        $Statuses->setProcessId($this->getProcessId());
    }
    
    public function end()
    {
        $Statuses = Statuses::getInstance();
        $Statuses->clean();
    }
    
    /**
     * @return dm_Demo
     */
    public function getDemo()
    {
        return $this->Demo;
    }
    
    public function setDemo(Demo $Demo)
    {
        $this->Demo = $Demo;
    }
    
    /**
     * @return dm_Log
     */
    public function getLog()
    {
        return $this->Log;
    }
}



