<?php

/**
 * @encoding     UTF-8
 * @package      WordPress Demo Content Manager
 * @link         https://codecanyon.net/item/wordpress-demo-content-manager/20669604
 * @copyright    Copyright (C) 2018 Torbara (http://torbara.com). All rights reserved.
 * @license      Envato Standard License http://themeforest.net/licenses/standard?ref=torbara
 * @author       Vladislav Dolgolenko <vladislavdolgolenko.com>
 * @support      <help@vladislavdolgolenko.com>
 */

namespace DM\Processing;

use DM\Processing\AbstractProcess;

class Installer extends AbstractProcess
{
    const PROCESS_ID = 'dm_Installer';
    
    protected $steps = array(
        '\DM\Processing\Steps\Start',
        '\DM\Processing\Steps\UploadDemoContent',
        '\DM\Processing\Steps\CreateDbTables',
        '\DM\Processing\Steps\ExtractDbTables',
        '\DM\Processing\Steps\ExtractMediafiles'
    );
    
    public function getProcessId()
    {
        return self::PROCESS_ID;
    }
    
    public function getProcessName()
    {
        return __('Installation of ', 'dm') . $this->getDemo()->getTitle();
    }
    
    /**
     * Starting or continuing processing of demo content installation
     * 
     * @throws Exception
     */
    public function processing()
    {
        $this->start();
        $this->getLog()->addLog('Start the installation of the demo content');
        $this->doSteps();
        $this->end();
    }
    
    /**
     * Aborting processing of demo content installation 
     * 
     * @return boolean
     */
    public function aborting()
    {
        $this->start();
        $this->getLog()->addLog('Start removing the demo content and restore the basic version of the content.');
        $this->doAborts();
        $this->end();
    }
}



