<?php

/**
 * @encoding     UTF-8
 * @package      WordPress Demo Content Manager
 * @link         https://codecanyon.net/item/wordpress-demo-content-manager/20669604
 * @copyright    Copyright (C) 2018 Torbara (http://torbara.com). All rights reserved.
 * @license      Envato Standard License http://themeforest.net/licenses/standard?ref=torbara
 * @author       Vladislav Dolgolenko <vladislavdolgolenko.com>
 * @support      <help@vladislavdolgolenko.com>
 */

namespace DM;

use DM\Log;
use DM\Manager;
use DM\Config;

if (!class_exists('\DM\Config')) {
    require_once 'Config.php';
}

class Service
{
    static public function changeWpTablePrefix($new_prefix)
    {
        if (Config::getInstance()->isMultiSite()) {
            return true;
        }
        
        $wp_filesystem = \DM\WPObjects\FileSystem\FileSystem::getInstance();
        
        $config_content = $wp_filesystem->get_contents(ABSPATH . 'wp-config.php');
        $patern  = '/(\$table_prefix.*)/';

        $config_content_changed = preg_replace($patern, '$table_prefix  = "'. $new_prefix .'";', $config_content);
        $result = $wp_filesystem->put_contents(ABSPATH . 'wp-config.php', $config_content_changed);
        if ($result) {
            Log::addLog('Changed the table prefix in wp-config configuration');
            return true;
        }

        throw new Exception('WpDemoManager: prefix not changed!');
    }
    
    /**
     *  Do not use wordpress librarys!!!
     */
    static public function getMultisiteCurrentTablePrefix($db_name)
    {
        $default_prefix = 'wp_';
        $file_path = Config::getInstance()->getDataPath();
        $json = file_get_contents( $file_path );
        if (!$json) {
            return $default_prefix;
        }
        $data = json_decode($json, true);
        
        if (!isset($data[$db_name])) {
            return $default_prefix;
        }
        
        $site_data = $data[$db_name];
        if (isset($site_data['current_table_prefix']) && $site_data['current_table_prefix']) {
            return $site_data['current_table_prefix'];
        }
        
        return $default_prefix;
    }
    
    static public function getDBname()
    {
        global $wpdb;
        return $wpdb->dbname;
    }
    
    static public function extractContent($from, $to)
    {
        if (is_dir($from)) {
            @wp_mkdir_p($to);
            $d = dir($from);
            while (false !== ($entry = $d->read())) {
                if ($entry == "." || $entry == "..") {
                    continue;
                }
                self::extractContent("$from/$entry", "$to/$entry");
            }
            $d->close();
        } else if (is_file($from)) { 
            copy($from, $to);
        }
    }
    
    static public function removeContent($from, $to)
    {
        $wp_filesystem = \DM\WPObjects\FileSystem\FileSystem::getInstance();

        if ($wp_filesystem->is_dir($from)) {
            $d = dir($from);
            while (false !== ($entry = $d->read())) {
                if ($entry == "." || $entry == "..") {
                    continue;
                }
                self::removeContent("$from/$entry", "$to/$entry");
            }
            $d->close();
        } else if ($wp_filesystem->is_file($from)) { 
            $wp_filesystem->delete($to);
        }
    }
    
    static public function getCurrentTheme($theme_name = null)
    {
        $current_theme = \wp_get_theme($theme_name);
        if (!is_object($current_theme) || !$current_theme->get('TextDomain')) {
            throw new \Exception('Undefined current template');
        }
        
        if ($current_theme->get('Template')) {
            return self::getCurrentTheme( $current_theme->get('Template') );
        }
        
        return $current_theme;
    }
    
    public static function getFreeSpace()
    {
        $bytes = disk_free_space( Config::getInstance()->getDemosDir() );
        if (!$bytes || !is_float($bytes)) {
            return false;
        }
        
        $mb = $bytes / ( 1024 * 1024 );
        return (int)$mb;
    }
    
    public static function getCurrentWordpressDbTables($prefix)
    {
        global $wpdb;
        $result = array();
        $all_db_tables = $wpdb->get_col("SHOW TABLES;");
        
        if (!is_array($all_db_tables) || !count($all_db_tables)) {
            throw new \Exception("Undefined current wordpress database tables");
        }
        
        foreach ($all_db_tables as $table_name) {
            if (preg_match("/$prefix/", $table_name) === 1) {
                $clear_table_name = str_replace($prefix, "", $table_name);
                $result[$clear_table_name] = $table_name;
            }
        }
        
        return $result;
    }
    
    public static function getInstalledDemoData($demo_id)
    {
        global $wpdb;
        
        $local_data = Manager::getInstance()->getLocalDemosDataById($demo_id);
        $remoute_data = Manager::getInstance()->getRemouteDemosDataById($demo_id);
        if (is_array($local_data)) {
            $origin_data = $local_data;
        } else if (is_array($remoute_data)) {
            $origin_data = $remoute_data;
        } else {
            $origin_data = array();
        }
        
        $demo_data = array_merge($origin_data, array('id' => $demo_id));
        $option_table_name = $demo_id . "options";
        
        // Template name
        $template_name = $wpdb->get_var("SELECT option_value FROM $option_table_name WHERE option_name = 'template'");
        $demo_data['template_name'] = $template_name;
        
        // Template version from local template
        $Theme = Service::getCurrentTheme($template_name);
        $demo_data['template_version'] = $Theme->Version;
        
        // Uploads url
        $upload_dir_information = \wp_upload_dir();
        $demo_data['uploads_url'] = $upload_dir_information['baseurl'];
        
        // Site url
        $current_site_url = \get_site_url();
        $demo_data['site_url'] = $current_site_url;
        
        // Screenshot
        if (!isset($demo_data['screenshot_url'])) {
            $screenshot_url = $wpdb->get_var("SELECT option_value FROM $option_table_name WHERE option_name = 'theme_mod_custom_logo'");
            if ($screenshot_url) {
                $demo_data['screenshot_url'] = $screenshot_url;
            }
        }
        
        // Name
        if (!isset($demo_data['name'])) {
            $demo_name = $wpdb->get_var("SELECT option_value FROM $option_table_name WHERE option_name = 'blogname'");
            $demo_data['name'] = $demo_name;
        }
        
        return $demo_data;
    }
    
    public static function moveSessionTokens($from_demo_id, $to_demo_id)
    {
        global $wpdb;
        $from_table_name = $from_demo_id . "usermeta";
        $session_tokens = $wpdb->get_var("SELECT meta_value FROM $from_table_name WHERE meta_key = 'session_tokens'");
        if (!$session_tokens) {
            return;
            //throw new \Exception('Current session tocken undefined');
        }
        
        $to_table_name = $to_demo_id . "usermeta";
        $wpdb->update( $to_table_name,
            array( 'meta_value' => $session_tokens),
            array( 'meta_key' => 'session_tokens' )
        );
    }
}
