<?php

/**
 * @encoding     UTF-8
 * @package      WPObjects
 * @link         https://github.com/VladislavDolgolenko/WPObjects
 * @copyright    Copyright (C) 2018 Vladislav Dolgolenko
 * @license      MIT License
 * @author       Vladislav Dolgolenko <vladislavdolgolenko.com>
 * @support      <help@vladislavdolgolenko.com>
 */

namespace DM\WPObjects\Factory\FilterHendler;

class WordPressContext implements
    \DM\WPObjects\EventManager\ListenerAggregateInterface
{
    /**
     * @var \DM\WPObjects\Factory\AbstractModelFactory
     */
    protected $Factory = null;
    
    /**
     * @var array
     */
    protected $filters = null;
    
    public function attach(\DM\WPObjects\EventManager\Manager $EventManager)
    {
        $EventManager->attach('set_query_filters', array($this, 'handler'));
    }
    
    public function detach(\DM\WPObjects\EventManager\Manager $EventManager)
    {
        $EventManager->detach('set_query_filters', array($this, 'handler'));
    }
    
    public function handler(\DM\WPObjects\Factory\AbstractModelFactory $Factory)
    {
        $this->Factory = $Factory;
        $this->filters = $Factory->getFilters();
        
        if (!isset($this->filters['page_context']) || !$this->filters['page_context']) {
            return;
        }
        
        global $post;
        $ModelType = $this->getModelType()->getContextModelType($post);
        if (is_null($ModelType)) {
            return;
        }
        
        $ContextModel = $ModelType->initModel($post);
        if ($ContextModel->getModelType()->getId() === $this->getFactory()->getModelType()->getId()) {
            $this->filters[$this->getFactory()->getIdAttrName()] = $ContextModel->getId();
        } else {
            $this->setUpContextToFilters($ContextModel);
        }
        
        $this->getFactory()->updateFilters($this->filters);
    }
    
    /**
     * @global \WP_Post $post
     * @return \DM\WPObjects\Model\AbstractTypicalModel
     */
    protected function getContextTypicalModel()
    {
        global $post;
        $ModelType = $this->getModelType()->getContextModelType($post);
        return $ModelType->initModel($post);
    }
    
    protected function setUpContextToFilters(\DM\WPObjects\Model\AbstractTypicalModel $ContextModel)
    {
        $attr = $this->getModelType()->getQualifierAttrName($ContextModel->getModelType()->getId());
        $this->filters[$attr] = $ContextModel->getId();
        $this->filters['id_not_ordered'] = true;
    }
    
    /**
     * @return \DM\WPObjects\Factory\AbstractModelFactory
     */
    protected function getFactory()
    {
        return $this->Factory;
    }
    
    /**
     * @return \DM\WPObjects\Model\AbstractModelType
     */
    protected function getModelType()
    {
        return $this->getFactory()->getModelType();
    }
    
}